#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "MibCompiler.h"
#include "AssignTYPE.h"
#include "symbol.h"


extern int yyErrorCnt;


/*
 * pName		name of TYPE definition
 * type			pointer to TYPE structure that details what
 *				is being named.
 * nLineNo		where the TYPE is defined in the MIB file.
 */
void ProcessType(char *pName,TYPE *type,int nLineNo)
 {
    SYMBOL *pSym;
    /*
     * This is simple; we just mark things as an alias. 'pName' is the
     * new name for type 'type'.
     */
    if( pSym = FindSymbol(pName,NEWTYPES) )
                 {
                    /*
                     * Symbol is already known! See if it is one of the 
                     * predefined types...
                     */
                    if( pSym->DefinedLineNo )
                        {
                            RestoreErrorMsg("Line %d : %s is an already known data type defined at line %d\n",pName,pSym->DefinedLineNo);
                            yyErrorCnt++;
                        }
                 }
            else {
                    /*
                     * We have a new one. So add it for later use.
                     */
                    pSym = (SYMBOL *) malloc( sizeof(SYMBOL) );
                    if( pSym )
                             {
                                /*
                                 * Save it.
                                 */
                                pSym->DefinedLineNo = nLineNo;
                                pSym->SymbolName = strdup(pName);
                                pSym->SymbolValue = (LIST *)type;
                                AddToSymbolTable(pSym,NEWTYPES);
                             }
                        else {
                                /*
                                 * No memory left!
                                 */
                                RestoreErrorMsg("Out of memory\n");
                             }
                 }
 }


void AddStandardTypes(void)
 {
    PAIR all0;
    /*
     * We add in any standard types that are created from basic level
     * types.
     */
    memset((char *)&all0,0,sizeof(all0));
    ProcessType("DisplayString",newTypeElement(OCTETTYPE,(char *)&all0),0);
    ProcessType("PhysAddress",newTypeElement(OCTETTYPE,(char *)&all0),0);
 }

