%{
#include <string.h>
#include <ctype.h>

#include "MibCompiler.h"
#include "symbol.h"
#include "y.tab.h"


#define	myyerror	yyerror
#undef	YYLMAX
#define	YYLMAX		8192

KEYTYPES eNoKeyWords = {ALLKEYWORDS};


struct table {
    char   *t_keyword;
    int	    t_value;
};

extern struct table reserved[];
extern struct table lowerReserved[];
%}

%%

"--"			{
                            int c, d;
			    for( d = 0; c = input(); d = c == '-' )
				if( (c == '\n') || (d && c == '-') ) break;
			}
[ \t]*			{
			}
\n			{
			}
"::="			{
			    return CCE;
			}
".."                    {
                            return DOTDOT;
                        }
";"			{
			    return SEMICOLON;
			}
","			{
			    return COMMA;
			}
"{"			{
			    return LBRACE;
			}
"}"			{
			    return RBRACE;
			}
"("			{
			    return LPAREN;
			}
")"			{
			    return RPAREN;
			}
[0-9]+			{
			    (void) sscanf (yytext, "%d", &yylval.yy_number);
			    return LITNUMBER;
			}
-[0-9]+			{
			    (void) sscanf (yytext, "%d", &yylval.yy_number);
			    return LITNUMBER;
			}
'[^'$]*'[BbHh]		{   register char *cp; register int i;

			    switch (*(cp = yytext + strlen (yytext) - 1)) {
				case 'H':
				case 'h':
				    *cp = NULL;
				    (void) sscanf (yytext + 1, "%x",
						&yylval.yy_number);
				    break;

				case 'B':
				case 'b':
				    *cp-- = NULL, *cp = NULL;
				    for (i = 0, cp = yytext + 1; *cp; ) {
					i <<= 1;
					i += *cp++ - '0';
				    }
				    yylval.yy_number = i;
				    break; 
			    }
			    return LITNUMBER;
			}
\"[^\"$]*\"		{
                            char *pF,*pT;
                            /*
                             * Process the literal string... we start
                             * by stripping out multiple WHITESPACE
                             * and replacing it with a single space.
                             */
			    yytext[strlen (yytext) - 1] = NULL;
			    yylval.yy_string = strdup(yytext + 1);
                            for( pF = pT = yylval.yy_string; *pF; )
                                if( isspace(*pF) && *(pF+1) && isspace(*(pF+1)) )
                                       {
                                            /*
                                             * We have multiple WHITESPACE. So
                                             * keep one as a BLANK.
                                             */
                                            *pF++;
                                       }
                                  else {
                                            /*
                                             * No. Save it away.
                                             */
                                            if( isspace(*pF) )
                                                     *pT++ = ' ';
                                                else *pT++ = *pF;
                                            pF++;
                                       }
                            *pT = '\0';
			    return LITSTRING;
			}
[A-Z][A-Za-z0-9-]*	{
                            /*
                             * See if we need to look for keywords.
                             */
                            int n;
                            SYMBOL *pSym;
                            if( eNoKeyWords == ALLKEYWORDS )
                                {
                                    /*
                                     * First, look for the unchanged keyword.
                                     * If we find it, cool. Just return the
                                     * type.
                                     */
                                    n = CompareKeywords(reserved,yytext,0);
                                    if( n >= 0 ) return(n);  

                                    /*
                                     * See if we have a case mismatch. If so
                                     * force an error!
                                     */
                                    n = CompareKeywords(reserved,yytext,1);
                                    if( n >= 0 ) return(ERROR);

                                    /*
                                     * Finally, check out the typedef types
                                     * of symbols.
                                     */
                                    pSym = FindSymbol(yytext,NEWTYPES);
                                    if( pSym )
					{
					 switch(((TYPE *)(pSym->SymbolValue))->Type)
						{
                                                    case NULLTYPE : return(NULLID);
                                                    case TICKTYPE : return(TIMETICKS);
                                                    case COUNTERTYPE : return(COUNTER);
                                                    case GAUGETYPE : return(GAUGE);
                                                    case IPADDRTYPE : return(IPADDRESS);
                                                    case NETWORKTYPE : return(NETWORKADDRESS);
                                                    case INTEGERTYPE : return(INTEGER);
                                                    case OCTETTYPE :   return(OCTSTR);
                                                    case STRINGTYPE : return(STRING);
						}
					}
                                }
			    yylval.yy_string = strdup(yytext);
			    return ID;
			}
[a-z][A-Za-z0-9-]*	{ 
                            int n;
                            if( eNoKeyWords ==  ALLKEYWORDS )
                                {
                                    /*
                                     * First, look for the unchanged keyword.
                                     */
                                    n = CompareKeywords(lowerReserved,yytext,0);
                                    if( n >= 0 ) return(n);
                                }
			    yylval.yy_string = strdup(yytext);
			    return NAME;
			}
.			{   
			    myyerror ("unknown token: \"%s\"", yytext);
			}

%%

struct table lowerReserved[] = {
    "deprecated", DEPRECATEDOBJECT,
    "mandatory", MANDATORYOBJECT,
    "obsolete", OBSOLETEOBJECT,
    "optional", OPTIONALOBJECT,

    "not-accessible", NOTACCESSIBLEOBJECT,
    "read-only", READONLYOBJECT,
    "read-write", READWRITEOBJECT,
    "write-only", WRITEONLYOBJECT,

    0,0
};

struct table reserved[] = {
    "ACCESS", ACCESS,
    "BEGIN", BGIN,
    "DEFINITIONS", DEFINITIONS,
    "DEFVAL", DEFVAL,
    "DESCRIPTION", DESCRIPTION,
    "END", END,
    "ENTERPRISE", ENTERPRISE,
    "EXPORTS", EXPORTS,
    "FROM", FROM,
    "IMPORTS", IMPORTS,
    "INDEX", INDEX,
    "OBJECT-TYPE", OBJECTTYPE,
    "OF", OF,
    "REFERENCE", REFERENCE,
    "SEQUENCE", SEQUENCE,
    "SIZE", SIZE,
    "STATUS", STATUS,
    "SYNTAX", SYNTAX,
    "TRAP-TYPE", TRAPTYPE,
    "VARIABLES", VARIABLES,

    "Counter", COUNTER,
    "Gauge", GAUGE,
    "INTEGER", INTEGER,
    "IpAddress", IPADDRESS,
    "NetworkAddress", NETWORKADDRESS,
    "NULL", NULLID,
    "OCTET", OCTET,
    "STRING", STRING,
    "OPAQUE", OPAQUE,
    "TimeTicks", TIMETICKS,
    "OBJECT", OBJECT,
    "IDENTIFIER", IDENTIFIER,

    0,0
};

static int stricmp(char *pS,char *pT)
 {
    for( ; *pS && *pT; pS++,pT++ )
        if( toupper(*pS) != toupper(*pT) ) break;
    return( *pS - *pT );
 }

static int CompareKeywords(struct table *pT,char *pS,int bNotCaseSensitive)
 {
    /*
     * Cycle through the table looking for a match.
     */
    for( ; pT->t_keyword; pT++ )
        {
            /*
             * See if we need to pay attention to the case or not.
             */
            if( bNotCaseSensitive & (stricmp(pT->t_keyword,pS) == 0) ||
                                             (strcmp(pT->t_keyword,pS) == 0) )
                                                     return(pT->t_value);
        }
    return(-1);
 }


char *ShouldBeKeyword(char *pS)
 {
    struct table *pT = reserved;
    /*
     * Cycle through the table looking for a match.
     */
    for( ; pT->t_keyword; pT++ )
        {
            /*
             * See if we need to pay attention to the case or not.
             */
            if( stricmp(pT->t_keyword,pS) == 0 ) return(pT->t_keyword);
        }
    return(0);
 }

